package com.hand.hls.dp.util;

import java.util.ArrayList;
import java.util.List;
import java.util.Objects;

import static java.util.Collections.emptyList;

/**
 * @author <a href="mailto:zhangnet14@gmail.com">Spencer Chang</a>
 * @date 2020/6/4 12:30
 * @since
 */
public class ArrayUtils {
    /**
     * Indicates whether the given object is not {@code null} and is an array.
     *
     * @param o the given object.
     * @return {@code true} if the given object is not {@code null} and is an array, otherwise {@code false}.
     */
    public static boolean isArray(Object o) {
        return o != null && o.getClass().isArray();
    }

    /**
     * Indicates whether the given array is {@code null} or empty.
     *
     * @param <T>   the type of elements of the array.
     * @param array the array to check.
     * @return {@code true} if the given array is {@code null} or empty, otherwise {@code false}.
     */
    public static <T> boolean isNullOrEmpty(T[] array) {
        return array == null || isEmpty(array);
    }

    /**
     * Returns an array containing the given arguments.
     *
     * @param <T>    the type of the array to return.
     * @param values the values to store in the array.
     * @return an array containing the given arguments.
     */
    @SafeVarargs
    public static <T> T[] array(T... values) {
        return values;
    }

    /**
     * Returns all the non-{@code null} elements in the given array.
     *
     * @param <T>   the type of elements of the array.
     * @param array the given array.
     * @return all the non-{@code null} elements in the given array. An empty list is returned if the given array is
     * {@code null}.
     */
    public static <T> List<T> nonNullElementsIn(T[] array) {
        if (array == null) {
            return emptyList();
        }
        List<T> nonNullElements = new ArrayList<>();
        for (T o : array) {
            if (o != null) {
                nonNullElements.add(o);
            }
        }
        return nonNullElements;
    }

    /**
     * Returns {@code true} if the given array has only {@code null} elements, {@code false} otherwise. If given array is
     * empty, this method returns {@code true}.
     *
     * @param <T>   the type of elements of the array.
     * @param array the given array. <b>It must not be null</b>.
     * @return {@code true} if the given array has only {@code null} elements or is empty, {@code false} otherwise.
     * @throws NullPointerException if the given array is {@code null}.
     */
    public static <T> boolean hasOnlyNullElements(T[] array) {
        Objects.requireNonNull(array);
        if (isEmpty(array)) {
            return false;
        }
        for (T o : array) {
            if (o != null) {
                return false;
            }
        }
        return true;
    }

    private static <T> boolean isEmpty(T[] array) {
        return array.length == 0;
    }

    public static boolean isObjectArray(Object o) {
        return isArray(o) && !isArrayTypePrimitive(o);
    }

    public static boolean isArrayTypePrimitive(Object o) {
        return o != null && o.getClass().getComponentType().isPrimitive();
    }
}
