package com.hand.hls.dp.util;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import java.io.File;
import java.io.FileNotFoundException;
import java.io.IOException;
import java.io.RandomAccessFile;
import java.nio.ByteBuffer;
import java.nio.channels.FileChannel;
import java.util.List;
import java.util.Objects;
import java.util.zip.ZipOutputStream;

/**
 * @author <a href="mailto:zhangnet14@gmail.com">Spencer Chang</a>
 * @date 2020/4/20 10:19
 * @since
 */
public class LogUtils {
    private static final Logger logger = LoggerFactory.getLogger(LogUtils.class);
    /**
     * 兼容不同操作系统间区别
     */
    private static final String separator = File.separator;
    /** 默认缓存大小 */
    public static final int DEFAULT_BUFFER_SIZE = 256;
    /** 默认大缓存大小 */
    public static final int DEFAULT_LARGE_BUFFER_SIZE = 1024;
    /** 数据流末尾 */
    public static final int EOF = -1;

    /**
     * 通过目录查找所属文件
     * @param dir 待查询目录
     * @param fileNames 查询结果
     */
    public static void findFiles(File dir, List<String> fileNames){
        Objects.requireNonNull(dir,"file dir must not be null");
        if (!dir.exists() || !dir.isDirectory()) {
            return;
        }
        String[] files = dir.list();
        for (String s : Objects.requireNonNull(files)) {
            File file = new File(dir, s);
            if (file.isFile()) {
                fileNames.add(dir + separator + file.getName());
            } else {
                findFiles(file, fileNames);
            }
        }
    }

    /**
     * 读取文件内容写入ZipOutputStream
     * @param filePath 待读取文件全路径
     * @param zos 写入内容的ZipOutputStream
     */
    public static void readFileToZip(final String filePath, ZipOutputStream zos){
        Objects.requireNonNull(filePath,"filePath must not be null");
        Objects.requireNonNull(zos,"ZipOutputStream must not be null");
        try (RandomAccessFile raf = new RandomAccessFile(filePath,"r");
             FileChannel fileChannel = raf.getChannel()){
            ByteBuffer buffer = ByteBuffer.allocate(DEFAULT_LARGE_BUFFER_SIZE);
            int n = 0;
            while (EOF != (n = fileChannel.read(buffer))) {
                // 切换buffer读模式
                buffer.flip();
                while (buffer.hasRemaining()) {
                    int remain = buffer.remaining();
                    byte[] bytes = new byte[remain];
                    // 从buffer读取数据到bytes
                    buffer.get(bytes);
                    // 内容写入ZipOutputStream
                    zos.write(bytes,0,n);
                    zos.flush();
                }
                // 切换buffer写模式
                buffer.clear();
            }
        } catch (FileNotFoundException fe){
            // 文件没找到
            if (logger.isErrorEnabled()) {
                logger.error("[{}] file not found！", filePath);
            }
        } catch (IOException e){
            // io例外
            if (logger.isWarnEnabled()) {
                logger.warn("[{}] file IOException！Message:[{}]", filePath, e.getMessage());
            }
        }
    }
}
